/*
 * Project name:
     clicker 2 for Kinetis (Simple 'Hello World' project)
 * Copyright:
     (c) Mikroelektronika, 2016.
 * Revision History:
     20140924:
       - initial release (FJ);
 * Description:
     This is a simple 'Hello World' project. It turns on/off LEDs connected to
     PTC18 and PTC19 pins depending on pressed buttons. Left button T2 changes
     mode of blinking and right button T3 changes frequency of blinking.
 * Test configuration:
     MCU:             MK64FN1M0VLL12
                      http://cache.nxp.com/files/microcontrollers/doc/data_sheet/K64P144M120SF5.pdf?fpsp=1&WT_TYPE=Data%20Sheets&WT_VENDOR=FREESCALE&WT_FILE_FORMAT=pdf&WT_ASSET=Documentation&fileExt=.pdf
     Dev.Board:       Clicker 2 for Kinetis
                      http://www.mikroe.com/kinetis/clicker-2/
     Oscillator:      PLL, 120.000MHz
     Ext. Modules:    None.
     SW:              mikroC PRO for ARM
                      http://www.mikroe.com/mikroc/arm/
 */

// pin definitions
sbit LD1 at PTC_PDOR.B18;
sbit LD2 at PTC_PDOR.B19;

// globals
char oldstate1 = 0, oldstate2 = 0;
char Example_State = 0;
char count;

//Timer2 : Timer Load Value Register = 0x03938700
//         Frequency                 = 120 MHz
//         Time                      = 500 ms
void InitTimer2(){
  SIM_SCGC6 |= (1 << PIT);       // Enable clock gating for Periodic interrupt timers
  PIT_MCR = 0x00;
  PIT_LDVAL2 = 0x03938700;       // Set Timer Load Value Register

  NVIC_IntEnable(IVT_INT_PIT2);  // Enable timer interrupts
  PIT_TCTRL2 = (1 << TIE);       // enable Timer 2 interrupts
  PIT_TCTRL2 |= (1 << TEN);      // start  Timer 2
}

// ISR
void Timer2_interrupt() iv IVT_INT_PIT2 {
char temp;
  PIT_TFLG2.TIF = 1;                      // clear timer interrupt flag
  // check T2 button state
  switch (Example_State & 0x0F){
    case 0 : LD1 = 0;                     // Both LEDs are OFF
             LD2 = 0;
             break;
    case 1 : LD1 ^= 1;                    // Only LD1 blinks
             LD2 = 0;
             break;
    case 2 : LD1 = 0;                     // Only LD2 blinks
             LD2 ^= 1;
             break;
    case 3 : LD1 ^= 1;                    // Both LEDs blinks alternately
             LD2  = !LD1;
             break;
    case 4 : LD1 ^= 1;                    // Both LEDs blink simultaneously
             LD2  = LD1;
             break;
    default : Example_State &= 0xF0;      // reset T2 state to zero
              break;
  }
  // check T3 button state
  switch (Example_State & 0xF0){
    case 0x00 : PIT_LDVAL2 = 0x03938700;  // Set Timer2 Interrupt time to 500ms
                break;
    case 0x10 : PIT_LDVAL2 = 0x02DC6C00;  // Set Timer2 Interrupt time to 400ms
                break;
    case 0x20 : PIT_LDVAL2 = 0x02255100;  // Set Timer2 Interrupt time to 300ms
                break;
    case 0x30 : PIT_LDVAL2 = 0x016E3600;  // Set Timer2 Interrupt time to 200ms
                break;
    case 0x40 : PIT_LDVAL2 = 0x00B71B00;  // Set Timer2 Interrupt time to 100ms
                break;
    default :   PIT_LDVAL2 = 0x03938700;  // Set Timer2 Interrupt time to 500ms
                Example_State &= 0x0F;    // reset T3 state to zero
                break;
  }
}

// main function
void main() {
  // Set PTD_PDIR pin 0 as digital input
  GPIO_Digital_Input(&PTD_PDIR, _GPIO_PINMASK_0);

  // Set PTD_PDIR pin 1 as digital input
  GPIO_Digital_Input(&PTD_PDIR, _GPIO_PINMASK_1);

  // Set PTC_PDOR pins 18 and 19 as digital output
  GPIO_Digital_Output(&PTC_PDOR, _GPIO_PINMASK_18 | _GPIO_PINMASK_19);

  LD1 = 0;                  // turn off LEDs
  LD2 = 0;

  Example_State = 0;        // set default Example state

  InitTimer2();             // initialize Timer2

  while(1){                 // Endless loop
    // check T2 button
    if (Button(&PTD_PDIR, 0, 1, 1)) {                // Detect logical zero
      oldstate1 = 1;                                 // Update flag
    }
    if (oldstate1 && Button(&PTD_PDIR, 0, 1, 0)) {   // Detect zero-to-one transition
      oldstate1 = 0;                                 // Update flag
      Example_State += 0x01;                         // set new Example state
      if ((Example_State & 0x0F) > 4)
        Example_State &= 0xF0;
    }
    // check T3 button
    if (Button(&PTD_PDIR, 1, 1, 1)) {                // Detect logical zero
      oldstate2 = 1;                                 // Update flag
    }
    if (oldstate2 && Button(&PTD_PDIR, 1, 1, 0)) {   // Detect zero-to-one transition
      oldstate2 = 0;                                 // Update flag
      Example_State += 0x10;                         // set new Example state
      if ((Example_State & 0xF0) > 0x40)
        Example_State &= 0x0F;
    }
  }
}